package com.sd.business.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import com.sd.business.entity.apply.po.ApplyBuy;
import com.sd.business.entity.apply.po.ApplyBuyBom;
import com.sd.business.entity.purchase.dto.PurchaseDto;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.purchase.po.PurchaseBom;
import com.sd.business.service.apply.ApplyBuyBomService;
import com.sd.business.service.apply.ApplyBuyService;
import com.sd.business.service.purchase.PurchaseBomService;
import com.sd.business.service.purchase.PurchaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 发起采购流程
 */
@Service
public class PurchaseFlow  extends FlowDelegate {

    @Autowired
    private PurchaseService purchaseService;

    @Autowired
    private PurchaseBomService purchaseBomService;

    @Autowired
    private ApplyBuyService applyBuyService;

    @Autowired
    private ApplyBuyBomService  applyBuyBomService;

    @Override
    public String getFlowKey() {
        return "purchase";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {

        PurchaseDto purchase = submitData.toJavaObject(PurchaseDto.class);
        purchase.setFlowStatus(FlowStatusEnum.IN_PROGRESS.getKey());
        purchase.setFlowId(flowId);

        if (purchase.getId() == null) {
            purchaseService.add(purchase);
        } else {
            purchaseService.edit(purchase);
        }

        return purchase.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {

        // 修改采购暂存表的数据
        Purchase purchase = purchaseService.getById(businessId);
        purchase.setFlowStatus(FlowStatusEnum.PASS.getKey());
        purchase.setFlowId(flowId);
        purchaseService.updateById(purchase);

        Map<Long, PurchaseBom> purchaseBomMap = purchaseBomService.mapKEntity(
                PurchaseBom::getApplyBuyBomId,
                q -> q.eq(PurchaseBom::getPurchaseId, businessId));

        List<ApplyBuyBom> applyBuyBomList = applyBuyBomService.list(
                q -> q.eq(ApplyBuyBom::getApplyBuyId, purchase.getApplyBuyId()));

        boolean isComplete = true;
        for (ApplyBuyBom applyBuyBom : applyBuyBomList) {
            PurchaseBom purchaseBom = purchaseBomMap.get(applyBuyBom.getId());
            if (purchaseBom != null) {
                applyBuyBom.setFrozenQuantity(applyBuyBom.getFrozenQuantity().subtract(purchaseBom.getPurchaseQuantity()));
                applyBuyBom.setPurchaseQuantity(applyBuyBom.getPurchaseQuantity().add(purchaseBom.getPurchaseQuantity()));
            }

            if (applyBuyBom.getQuantity().compareTo(applyBuyBom.getPurchaseQuantity()) > 0) {
                isComplete = false;
            }
        }

        applyBuyBomService.updateBatchById(applyBuyBomList);
        if (isComplete) {
            applyBuyService.update(q -> q
                    .eq(BaseIdPo::getId, purchase.getApplyBuyId())
                    .set(ApplyBuy::getFlowStatus, "4")
                    .set(BasePo::getUpdateTime, new Date())
                    .set(BasePo::getUpdateUser, SecurityUtils.getUserId()));
        }

    }

    @Override
    public void returnToOriginator(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        purchaseService.update(q -> q
                .eq(BaseIdPo::getId, businessId)
                .set(Purchase::getFlowStatus, flowStatus.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        PurchaseDto purchase = submitData.toJavaObject(PurchaseDto.class);
        purchase.setFlowStatus(FlowStatusEnum.IN_PROGRESS.getKey());
        purchaseService.edit(purchase);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {

        Map<Long, PurchaseBom> purchaseBomMap = purchaseBomService.mapKEntity(
                PurchaseBom::getApplyBuyBomId,
                q -> q.eq(PurchaseBom::getPurchaseId, businessId));

        List<ApplyBuyBom> applyBuyBomList = applyBuyBomService.list(
                q -> q.eq(ApplyBuyBom::getApplyBuyId, businessId));

        applyBuyBomList.forEach(item -> {
            PurchaseBom purchaseBom = purchaseBomMap.get(item.getId());
            if (purchaseBom == null) {
                return;
            }
            item.setFrozenQuantity(item.getFrozenQuantity().subtract(purchaseBom.getPurchaseQuantity()));
        });
        applyBuyBomService.updateBatchById(applyBuyBomList);

        purchaseService.update(q -> q
                .eq(BaseIdPo::getId, businessId)
                .set(Purchase::getFlowStatus, flowStatus.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    /**
     * 作废
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     */
    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        reject(flowId, businessId, flowStatus);
    }

}
