package com.sd.business.service.artwork.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.entity.ObsFile;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.artwork.dto.ArtworkLibraryDto;
import com.sd.business.entity.artwork.dto.ArtworkLibrarySelectDto;
import com.sd.business.entity.artwork.po.ArtworkLibrary;
import com.sd.business.entity.artwork.vo.ArtworkLibraryVo;
import com.sd.business.mapper.artwork.ArtworkLibraryMapper;
import com.sd.business.service.artwork.ArtworkLibraryService;
import com.sd.framework.util.Assert;
import org.springframework.stereotype.Service;

import java.util.Collections;

/**
 * <p>
 * 图稿库 服务实现类
 * </p>
 *
 * @author
 * @since 2023-08-02
 */
@Service
public class ArtworkLibraryServiceImpl extends ServiceImpl<ArtworkLibraryMapper, ArtworkLibrary> implements ArtworkLibraryService {

    /**
     * 分页
     */
    @Override
    public Page<ArtworkLibraryVo> getPage(ArtworkLibrarySelectDto dto) {
        IWrapper<ArtworkLibrary> wrapper = getWrapper();
        wrapper.like("al", ArtworkLibrary::getArtworkName, dto.getArtworkName());
        wrapper.eq("al", ArtworkLibrary::getDepartmentId, dto.getDepartmentId());
        wrapper.orderByDesc("al", ArtworkLibrary::getId);
        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    /**
     * 数据回显
     */
    @Override
    public ArtworkLibraryVo detail(Long id) {
        ArtworkLibrary ArtworkLibrary = this.getById(id);
        return BeanUtil.toBean(ArtworkLibrary, ArtworkLibraryVo.class);
    }

    /**
     * 新增图稿
     */
    @DSTransactional
    @Override
    public void add(ArtworkLibraryDto artworkLibraryDto) {
        // 判断图稿名称是否存在
        long count = count(q -> q
                .eq(ArtworkLibrary::getArtworkName, artworkLibraryDto.getArtworkName())
                .last("limit 1"));
        Assert.eqZero(count, "图稿名称已存在");

        save(artworkLibraryDto);

        ObsFile imgFile = new ObsFile();
        imgFile.setId(artworkLibraryDto.getImgId());

        ObsFile fileFile = new ObsFile();
        fileFile.setId(artworkLibraryDto.getFileId());

        ObsFileUtil.saveFile(Collections.singletonList(imgFile), artworkLibraryDto.getId(), 1);
        ObsFileUtil.saveFile(Collections.singletonList(fileFile), artworkLibraryDto.getId(), 2);
    }

    /**
     * 修改图稿
     */
    @DSTransactional
    @Override
    public void edit(ArtworkLibraryDto artworkLibraryDto) {
        // 判断修改的稿名是否已经存在
        long count = count(q -> q
                .eq(ArtworkLibrary::getArtworkName, artworkLibraryDto.getArtworkName())
                .ne(BaseIdPo::getId, artworkLibraryDto.getId())
                .last("limit 1"));
        Assert.eqZero(count, "图稿名称已存在");

        updateById(artworkLibraryDto);

        ObsFile imgFile = new ObsFile();
        imgFile.setId(artworkLibraryDto.getImgId());

        ObsFile fileFile = new ObsFile();
        fileFile.setId(artworkLibraryDto.getFileId());

        ObsFileUtil.editFile(Collections.singletonList(imgFile), artworkLibraryDto.getId(), 1);
        ObsFileUtil.editFile(Collections.singletonList(fileFile), artworkLibraryDto.getId(), 2);
    }

    /**
     * 删除图稿
     */
    @DSTransactional
    @Override
    public void delete(Long id) {
        removeById(id);
        ObsFileUtil.removeFile(id);
    }

}
