package com.sd.business.service.bom.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.TreeUtil;
import com.sd.business.entity.bom.dto.BomClassifyDto;
import com.sd.business.entity.bom.dto.BomClassifySelectDto;
import com.sd.business.entity.bom.po.BomClassify;
import com.sd.business.entity.bom.vo.BomClassifyVo;
import com.sd.business.mapper.bom.BomClassifyMapper;
import com.sd.business.service.bom.BomClassifyService;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;


/**
 * <p>
 * bom分类 服务实现类
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@Service
public class BomClassifyServiceImpl extends ServiceImpl<BomClassifyMapper, BomClassify> implements BomClassifyService {

    @Override
    public List<JSONObject> tree(BomClassifySelectDto dto) {
        List<BomClassify> list = list(q -> q
                .like(ObjectUtil.isNotEmpty(dto.getName()), BomClassify::getName, dto.getName()));
        return TreeUtil.buildTreeObj(list);
    }

    @Override
    public BomClassifyVo detail(Long id) {
        BomClassify bomClassify = this.getById(id);
        BomClassifyVo result = BeanUtil.toBean(bomClassify, BomClassifyVo.class);
        return result;
    }

    @Override
    public void add(BomClassifyDto bomClassifyDto) {
        long count = count(q -> q
                .eq(BomClassify::getCode, bomClassifyDto.getCode())
                .last("limit 1")
        );
        if (count > 0) {
            throw new ServiceException("分类编码已存在");
        }
        this.save(bomClassifyDto);
    }

    @Override
    public void edit(BomClassifyDto bomClassifyDto) {
        long count = count(q -> q
                .eq(BomClassify::getCode, bomClassifyDto.getCode())
                .ne(BaseIdPo::getId, bomClassifyDto.getId())
                .last("limit 1")
        );
        if (count > 0) {
            throw new ServiceException("分类编码已存在");
        }
        this.updateById(bomClassifyDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public List<Long> getChildrenIdList(Long parentId) {
        List<BomClassify> list = list();
        List<Long> childrenIdList = new ArrayList<>();
        getChildrenIdList(parentId, childrenIdList, list);
        return childrenIdList;
    }

    @Override
    public List<Long> getChildrenIdList(List<Long> parentIdList) {
        List<BomClassify> list = list();
        List<Long> childrenIdList = new ArrayList<>();
        parentIdList.forEach(item -> getChildrenIdList(item, childrenIdList, list));
        return childrenIdList;
    }

    /**
     * 递归添加子集节点id
     */
    private void getChildrenIdList(Long parentId, List<Long> childrenIdList, List<BomClassify> list) {
        childrenIdList.add(parentId);
        for (BomClassify bomClassify : list) {
            if (Objects.equals(bomClassify.getParentId(), parentId)) {
                getChildrenIdList(bomClassify.getId(), childrenIdList, list);
            }
        }
    }

}
