package com.sd.business.service.inventory;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.service.BaseService;
import com.sd.business.entity.inventory.bo.InOutFun;
import com.sd.business.entity.inventory.dto.InventorySelectDto;
import com.sd.business.entity.inventory.dto.QuantityByWarehouseDto;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.inventory.vo.InventoryVo;
import com.sd.business.entity.inventory.vo.QuantityByDepartmentVo;
import com.sd.business.entity.inventory.vo.QuantityByWarehouseVo;

import java.util.List;


/**
 * <p>
 * 库存 服务类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
public interface InventoryService extends BaseService<Inventory> {

    /**
     * 库存分页
     */
    Page<InventoryVo> getPage(InventorySelectDto dto, String tableName);

    /**
     * 根据仓库获取库存数量
     */
    List<QuantityByWarehouseVo> getQuantityByWarehouse(QuantityByWarehouseDto dto);

    /**
     * 根据事业部获取库存数量
     */
    List<QuantityByDepartmentVo> getQuantityByDepartment();

    /**
     * 入库逻辑
     *
     * @param departmentId 事业部id
     * @param warehouseId  仓库id
     * @param list         入库明细
     */
    void in(Long departmentId, Long warehouseId, List<? extends InOutFun> list);

    /**
     * 出库逻辑
     *
     * @param departmentId 事业部id
     * @param warehouseId  仓库id
     * @param list         出库明细
     */
    void out(Long departmentId, Long warehouseId, List<? extends InOutFun> list);

}
