package com.sd.business.service.order.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.LogicIgnore;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderSelectDto;
import com.sd.business.entity.order.dto.OrderSkuDto;
import com.sd.business.entity.order.dto.SkuSpecPriceDto;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.*;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.order.vo.OrderSkuBomVo;
import com.sd.business.entity.order.vo.OrderSkuVo;
import com.sd.business.entity.order.vo.SkuSpecPriceVo;
import com.sd.business.entity.price.po.PriceBillingStandardDetail;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.order.OrderMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.order.*;
import com.sd.business.service.price.PriceBillingStandardDetailService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 订单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, OrderInfo> implements OrderService {

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Autowired
    private OrderOperatingLogService orderOperatingLogService;

    @Autowired
    private PriceBillingStandardDetailService priceBillingStandardDetailService;

    @Override
    public Page<OrderInfoVo> getPage(OrderSelectDto dto) {
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.orderByDesc("o", OrderInfo::getId);
        wrapper.eq("d", Department::getId, dto.getDepartmentId());
        wrapper.like("d", Department::getName, dto.getDepartmentName());
        wrapper.like("o", OrderInfo::getCode, dto.getCode());
        wrapper.like("o", OrderInfo::getWlnCode, dto.getWlnCode());
        wrapper.eq("o", OrderInfo::getStatus, dto.getStatus());
        wrapper.eq("o", OrderInfo::getSettlementStatus, dto.getSettlementStatus());
        wrapper.ge("o", OrderInfo::getWlnCreateTime, dto.getBeginTime());
        wrapper.le("o", OrderInfo::getWlnCreateTime, dto.getEndTime());

        if (Objects.equals(dto.getLinkedStatementOfAccount(), 0)) {
            wrapper.isNull("o.statement_of_account_id");
            wrapper.in("o", OrderInfo::getStatus,
                    OrderStatusEnum.COMPLETION_PRODUCTION.getKey(),
                    OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        } else if (Objects.equals(dto.getLinkedStatementOfAccount(), 1)) {
            wrapper.isNotNull("o.statement_of_account_id");
            wrapper.in("o", OrderInfo::getStatus,
                    OrderStatusEnum.COMPLETION_PRODUCTION.getKey(),
                    OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        }

        if (dto.getException() != null) {
            if (dto.getException().equals(StatusConstant.YES)) {
                wrapper.ne("o", OrderInfo::getExceptionType, "0");
            } else {
                wrapper.eq("o", OrderInfo::getExceptionType, "0");
            }
        }

        Page<OrderInfoVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<OrderInfoVo> records = page.getRecords();

        if (records.size() == 0) {
            return page;
        }

        for (OrderInfoVo record : records) {

            // 设置异常提示
            String exceptionType = record.getExceptionType();
            if (exceptionType.equals("0")) {
                continue;
            }
            String exceptionTypeDetail = Arrays.stream(exceptionType.split(","))
                    .map(Convert::toInt)
                    .map(OrderExceptionTypeEnum::getEnumValue)
                    .collect(Collectors.joining(","));
            record.setExceptionTypeDetail(exceptionTypeDetail);
        }

        return page;
    }

    @Override
    public OrderInfoVo detail(Long id) {
        OrderInfo orderInfo = this.getById(id);
        if (orderInfo == null) {
            throw new ServiceException("未知订单");
        }

        // 订单
        OrderInfoVo result = BeanUtil.toBean(orderInfo, OrderInfoVo.class);

        // 赋值事业部名称
        Department department = departmentService.getById(orderInfo.getDepartmentId());
        if (department != null) {
            result.setDepartmentName(department.getName());
        }

        // 赋值订单明细
        List<OrderSku> tempOrderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuVo> orderSkuList = BeanUtil.copyToList(tempOrderSkuList, OrderSkuVo.class);
        skuSpecService.attributeAssign(orderSkuList, OrderSkuVo::getSkuSpecId, (item, skuSpec) -> {
            item.setName(skuSpec.getName());
            item.setCode(skuSpec.getCode());
        });
        bomSpecService.attributeAssign(orderSkuList, OrderSkuVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomName(bomSpec.getName());
            item.setBomCode(bomSpec.getCode());
        });
        result.setOrderSkuList(orderSkuList);

        // 赋值包材配件/单品
        List<OrderSkuBom> tempOrderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));
        List<OrderSkuBomVo> orderSkuBomList = BeanUtil.copyToList(tempOrderSkuBomList, OrderSkuBomVo.class);
        bomSpecService.attributeAssign(orderSkuBomList, OrderSkuBom::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecName(bomSpec.getName());
        });
        Map<Long, List<OrderSkuBomVo>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        for (OrderSkuVo orderSkuVo : orderSkuList) {
            orderSkuVo.setOrderSkuBomList(orderSkuBomMap.getOrDefault(orderSkuVo.getId(), Collections.emptyList()));
        }

        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(OrderInfoDto orderDto) {
        orderDto.setSource(1);
        orderDto.setCode("PI" + new Date().getTime());
        orderDto.setSettlementStatus(1);
        orderDto.setTaxRate(new BigDecimal("10.00"));
        orderDto.setTag("0");
        this.save(orderDto);

        List<OrderSkuDto> orderSkuList = orderDto.getOrderSkuList();
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(orderDto.getId()))
                .peek(item -> item.setStockPreparationStatus(StatusConstant.NO))
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.saveBatch(tempOrderSkuList);

        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(orderDto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.saveBatch(orderSkuBomList);

        OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
        orderOperatingLog.setType(10);
        orderOperatingLog.setOrderId(orderDto.getId());
        orderOperatingLog.setOrderCode(orderDto.getCode());
        orderOperatingLogService.save(orderOperatingLog);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(OrderInfoDto orderDto) {
        this.updateById(orderDto);

        List<OrderSkuDto> orderSkuList = orderDto.getOrderSkuList();
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(orderDto.getId()))
                .peek(item -> item.setStockPreparationStatus(StatusConstant.NO))
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.editLinked(tempOrderSkuList, OrderSku::getOrderId, orderDto.getId());

        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(orderDto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.editLinked(orderSkuBomList, OrderSkuBom::getOrderId, orderDto.getId());

        if (orderDto.getUpdateType() != null) {
            OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
            orderOperatingLog.setType(orderDto.getUpdateType());
            orderOperatingLog.setOrderId(orderDto.getId());
            orderOperatingLog.setOrderCode(orderDto.getCode());
            orderOperatingLogService.save(orderOperatingLog);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Long id) {
        OrderInfo orderInfo = getById(id);

        if (orderInfo.getStatus() > OrderStatusEnum.UNDER_REVIEW.getKey()) {
            throw new ServiceException("订单已确认，无法直接删除，需要走审批流程");
        }

        this.removeById(id);
        orderSkuService.remove(q -> q.eq(OrderSku::getOrderId, id));
        orderSkuBomService.remove(q -> q.eq(OrderSkuBom::getOrderId, id));
        orderPackageBomService.remove(q -> q.eq(OrderPackageBom::getOrderId, id));

        OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
        orderOperatingLog.setType(30);
        orderOperatingLog.setOrderId(id);
        orderOperatingLog.setOrderCode(orderInfo.getCode());
        orderOperatingLogService.save(orderOperatingLog);
    }

    @Override
    public SkuSpecPriceVo getSkuSpecPrice(SkuSpecPriceDto dto) {

        // 查询单价
        Long skuSpecId = dto.getSkuSpecId();
        SkuSpec skuSpec = skuSpecService.getById(skuSpecId);
        Assert.notNull(skuSpec, "未知sku规格");
        Long bomSpecId = skuSpec.getBomSpecId();
        Assert.notNull(bomSpecId, "sku未绑定bom");
        BomSpec bomSpec = bomSpecService.getById(bomSpecId);
        Assert.notNull(bomSpec, "未知bom规格");

        // 其他价格查询
        Long deptId = SecurityUtils.getDeptId();
        Department department = departmentService.getById(deptId);
        Assert.notNull(department, "未知事业部");
        Long priceBillingStandardId = department.getPriceBillingStandardId();
        Assert.notNull(priceBillingStandardId, "事业部未绑定加工计费标准");
        List<PriceBillingStandardDetail> priceBillingStandardDetailList = priceBillingStandardDetailService.list(
                q -> q.eq(PriceBillingStandardDetail::getPriceBillingStandardId, priceBillingStandardId));

        // 赋值
        SkuSpecPriceVo skuSpecPriceVo = new SkuSpecPriceVo();
        skuSpecPriceVo.setUnitPrice(bomSpec.getInternalSellingPrice());
        skuSpecPriceVo.setCustomProcessingFee(BigDecimal.ZERO);
        skuSpecPriceVo.setLssueFee(BigDecimal.ZERO);
        skuSpecPriceVo.setDeliveryMaterialsFee(BigDecimal.ZERO);
        skuSpecPriceVo.setPackingLabor(BigDecimal.ZERO);

        for (PriceBillingStandardDetail priceBillingStandardDetail : priceBillingStandardDetailList) {
            String chargeItem = priceBillingStandardDetail.getChargeItem();
            BigDecimal chargePrice = ObjectUtil.defaultIfNull(priceBillingStandardDetail.getChargePrice(), BigDecimal.ZERO);
            switch (chargeItem) {
                case "40":
                    skuSpecPriceVo.setPackingLabor(chargePrice);
                    break;
                case "50":
                    skuSpecPriceVo.setLssueFee(chargePrice);
                    break;
                case "60":
                    skuSpecPriceVo.setDeliveryMaterialsFee(chargePrice);
                    break;
                default:
                    if (Objects.equals(chargeItem, skuSpec.getMachinedPanel())
                            && dto.getQuantity().compareTo(priceBillingStandardDetail.getSectionMin()) >= 0
                            && dto.getQuantity().compareTo(priceBillingStandardDetail.getSectionMax()) < 0) {
                        skuSpecPriceVo.setCustomProcessingFee(chargePrice);
                        skuSpecPriceVo.setCustomProcessingType(chargeItem);
                    }
            }
        }

        return skuSpecPriceVo;
    }

    @Override
    public String addExceptionType(String exceptionTypeStr, Integer addExceptionType) {
        if (StrUtil.isBlank(exceptionTypeStr) || exceptionTypeStr.equals("0")) {
            return addExceptionType.toString();
        }

        if (Arrays.asList(exceptionTypeStr.split(",")).contains(addExceptionType.toString())) {
            return exceptionTypeStr;
        }

        return exceptionTypeStr + "," + addExceptionType;
    }

    @LogicIgnore
    @Override
    public Page<OrderInfoVo> deletedOrderPage(OrderSelectDto dto) {
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.orderByDesc("o", OrderInfo::getId);
        wrapper.eq("d", Department::getId, dto.getDepartmentId());
        wrapper.like("o", OrderInfo::getCode, dto.getCode());
        wrapper.like("o", OrderInfo::getWlnCode, dto.getWlnCode());
        wrapper.eq("o", OrderInfo::getDelFlag, StatusConstant.DELETED);

        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @LogicIgnore
    @Override
    public OrderInfoVo deletedOrderDetail(BaseSelectDto dto) {
        return detail(dto.getId());
    }

}
