package com.sd.business.service.production.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderPackageBom;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.production.dto.ProductionOrderDto;
import com.sd.business.entity.production.dto.RapidPackagingDto;
import com.sd.business.entity.production.dto.SetTagDto;
import com.sd.business.entity.production.po.ProductionTask;
import com.sd.business.entity.production.vo.ProductionOrderDetailVo;
import com.sd.business.entity.production.vo.ProductionOrderScheduleVo;
import com.sd.business.entity.production.vo.ProductionOrderVo;
import com.sd.business.mapper.production.ProductionOrderMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.order.OrderPackageBomService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.ProductionOrderService;
import com.sd.business.service.production.ProductionTaskService;
import com.sd.business.service.sku.SkuService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class ProductionOrderServiceImpl implements ProductionOrderService {

    @Autowired
    private ProductionOrderMapper productionOrderMapper;

    @Autowired
    private ProductionTaskService productionTaskService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private SkuService skuService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Override
    public Page<ProductionOrderVo> getPage(ProductionOrderDto dto) {

        IWrapper<ProductionOrderVo> wrapper = IWrapper.getWrapper();
        wrapper.ge("oi", OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey());
        wrapper.like("oi", OrderInfo::getCode, dto.getCode());
        wrapper.like("oi", OrderInfo::getWlnCode, dto.getWlnCode());
        wrapper.eq("oi", OrderInfo::getDepartmentId, dto.getDepartmentId());
        wrapper.in("oi", OrderInfo::getTag, dto.getTagList());
        wrapper.ge("oi", OrderInfo::getDeliveryTime, dto.getBeginTime());
        wrapper.le("oi", OrderInfo::getDeliveryTime, dto.getEndTime());
        wrapper.eq("oi", OrderInfo::getStatus, dto.getStatus());

        Page<ProductionOrderVo> page = productionOrderMapper.getPage(dto.getPage(), wrapper);
        List<ProductionOrderVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        // 赋值进度
        List<Long> orderIdList = records.stream().map(ProductionOrderVo::getOrderId).collect(Collectors.toList());
        List<ProductionTask> productionTaskList = productionTaskService.list(q -> q.in(ProductionTask::getOrderId, orderIdList));

        List<ProductionOrderScheduleVo> productionOrderScheduleVoList = productionTaskList.stream()
                .map(item -> ProductionOrderScheduleVo.builder()
                        .orderId(item.getOrderId())
                        .skuSpecId(item.getSkuSpecId())
                        .productionQuantity(item.getProductionQuantity())
                        .completeQuantity(item.getCompleteQuantity())
                        .build())
                .collect(Collectors.toList());

        skuSpecService.attributeAssign(productionOrderScheduleVoList, ProductionOrderScheduleVo::getSkuSpecId,
                (item, skuSpec) -> {
                    item.setSkuId(skuSpec.getSkuId());
                    item.setSkuSpecCode(skuSpec.getCode());
                    item.setSkuSpecName(skuSpec.getName());
                });

        skuService.attributeAssign(productionOrderScheduleVoList, ProductionOrderScheduleVo::getSkuId,
                (item, sku) -> {
                    item.setSkuName(sku.getName());
                });

        Map<Long, List<ProductionOrderScheduleVo>> productionOrderScheduleVoMap = productionOrderScheduleVoList
                .stream().collect(Collectors.groupingBy(ProductionOrderScheduleVo::getOrderId));

        for (ProductionOrderVo record : records) {
            record.setProductionOrderScheduleVoList(
                    productionOrderScheduleVoMap.getOrDefault(record.getOrderId(), Collections.emptyList()));
        }

        return page;
    }

    @Override
    public void setTag(SetTagDto dto) {
        orderService.update(q -> q.in(BaseIdPo::getId, dto.getOrderIdList()).set(OrderInfo::getTag, dto.getTag()));
    }

    @Override
    public ProductionOrderDetailVo detail(BaseSelectDto dto) {
        Long id = dto.getId();
        Assert.notNull(id, "订单id不能为空");

        List<ProductionOrderDetailVo.SkuSpec> skuSpecList = getSkuSpecList(id);
        List<ProductionOrderDetailVo.OrderPackage> packageList = getPackageList(id);

        ProductionOrderDetailVo vo = new ProductionOrderDetailVo();
        vo.setSkuSpecList(skuSpecList);
        vo.setPackageList(packageList);
        return vo;
    }

    @Override
    public synchronized void rapidPackaging(RapidPackagingDto dto) {
        Long orderId = dto.getOrderId();
        OrderInfo orderInfo = orderService.getById(orderId);
        if (!orderInfo.getStatus().equals(OrderStatusEnum.IN_PRODUCTION.getKey())) {
            throw new ServiceException("订单已被处理");
        }

        List<OrderPackageBom> orderPackageBomList = dto.getPackageList();
        orderPackageBomList.forEach(item -> item.setOrderId(orderId));
        orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, orderId);

        Date date = new Date();
        orderService.update(q -> q
                .eq(BaseIdPo::getId, orderId)
                .set(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey())
                .set(OrderInfo::getShippingTime, date)
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, date));
    }

    private List<ProductionOrderDetailVo.SkuSpec> getSkuSpecList(Long orderId) {

        // 获取订单sku规格
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderId));

        // 相同订单sku规格合并，数量相加
        List<ProductionOrderDetailVo.SkuSpec> skuSpecList = new ArrayList<>(
                orderSkuList.stream()
                        .map(item -> {
                            ProductionOrderDetailVo.SkuSpec skuSpec = new ProductionOrderDetailVo.SkuSpec();
                            skuSpec.setSkuSpecId(item.getSkuSpecId());
                            skuSpec.setQuantity(item.getQuantity());
                            return skuSpec;
                        })
                        .collect(Collectors.toMap(
                                ProductionOrderDetailVo.SkuSpec::getSkuSpecId,
                                Function.identity(),
                                (v1, v2) -> {
                                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                                    return v1;
                                })
                        ).values());

        // 赋值订单sku规格品号以及品名
        skuSpecService.attributeAssign(skuSpecList, ProductionOrderDetailVo.SkuSpec::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecName(skuSpec.getName());
            item.setSkuSpecCode(skuSpec.getCode());
        });

        return skuSpecList;
    }

    private List<ProductionOrderDetailVo.OrderPackage> getPackageList(Long orderId) {

        List<OrderPackageBom> list = orderPackageBomService.list(q -> q.eq(OrderPackageBom::getOrderId, orderId));

        List<ProductionOrderDetailVo.OrderPackage> orderPackageList = list.stream().map(item -> {
            ProductionOrderDetailVo.OrderPackage orderPackage = new ProductionOrderDetailVo.OrderPackage();
            orderPackage.setBomSpecId(item.getBomSpecId());
            orderPackage.setQuantity(item.getQuantity());
            orderPackage.setCostPrice(item.getCostPrice());
            orderPackage.setInternalSellingPrice(item.getInternalSellingPrice());
            return orderPackage;
        }).collect(Collectors.toList());

        // 赋值品名 品号 销售单价 成本价
        bomSpecService.attributeAssign(orderPackageList, ProductionOrderDetailVo.OrderPackage::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        return orderPackageList;
    }

}
