package com.sd.business.service.sku.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.sku.dto.SkuSpecDto;
import com.sd.business.entity.sku.dto.SkuSpecSelectDto;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.vo.SkuSpecVo;
import com.sd.business.mapper.sku.SkuSpecMapper;
import com.sd.business.service.sku.SkuSpecService;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * sku规格 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-05
 */
@Service
public class SkuSpecServiceImpl extends ServiceImpl<SkuSpecMapper, SkuSpec> implements SkuSpecService {

    @Override
    public Page<SkuSpecVo> getPage(SkuSpecSelectDto dto) {
        IWrapper<SkuSpec> wrapper = getWrapper();
        wrapper.orderByDesc("ss", SkuSpec::getId);
        Page<SkuSpecVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public SkuSpecVo detail(Long id) {
        SkuSpec SkuSpec = this.getById(id);
        SkuSpecVo result = BeanUtil.toBean(SkuSpec, SkuSpecVo.class);
        return result;
    }

    @Override
    public void add(SkuSpecDto skuSpecDto) {
        this.save(skuSpecDto);
    }

    @Override
    public void edit(SkuSpecDto skuSpecDto) {
        this.updateById(skuSpecDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public Map<Long, BomSpecBo> getBomBoByIdList(List<Long> bomSpecIdList) {

        if (ObjectUtil.isEmpty(bomSpecIdList)) {
            return Collections.emptyMap();
        }

        bomSpecIdList = bomSpecIdList.stream().filter(ObjectUtil::isNotNull).distinct().collect(Collectors.toList());

        if (ObjectUtil.isEmpty(bomSpecIdList)) {
            return Collections.emptyMap();
        }

        List<BomSpecBo> list = baseMapper.getBomBoList(Wrappers.query().in("bs.id", bomSpecIdList));

        return list.stream().collect(Collectors.toMap(BomSpecBo::getBomSpecId, Function.identity()));
    }

}
