package com.sd.business.service.statement.impl;

import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.metadata.data.WriteCellData;
import com.alibaba.excel.write.handler.CellWriteHandler;
import com.alibaba.excel.write.metadata.holder.WriteSheetHolder;
import com.alibaba.excel.write.metadata.holder.WriteTableHolder;
import com.sd.business.entity.statement.vo.DocumentByOrderVo;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.util.CellRangeAddress;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class DocumentByOrderExcelCellMergeStrategy implements CellWriteHandler {

    // 无需合并的行数
    private final int mergeRowIndex = 3;
    // 订单无需合并的行
    private final List<Integer> orderRowIndex = new ArrayList<>();
    // 订单需要合并的列
    private final List<Integer> orderColIndex = Arrays.asList(0, 1, 2);
    // sku无需合并的行
    private final List<Integer> skuRowIndex = new ArrayList<>();
    // sku需要合并的列
    private final List<Integer> skuColIndex = Arrays.asList(3, 4, 5, 15, 16, 17, 18, 19);

    public DocumentByOrderExcelCellMergeStrategy(List<DocumentByOrderVo> documentByOrderVoList) {
        for (DocumentByOrderVo documentByOrderVo : documentByOrderVoList) {

            List<DocumentByOrderVo.SkuSpec> skuSpecList = documentByOrderVo.getSkuSpecList();

            // 赋值订单无需合并的行
            Integer size = skuSpecList.stream().map(item -> item.getBomSpecList().size()).reduce(0, Integer::sum);
            if (orderRowIndex.size() == 0) {
                orderRowIndex.add(size + mergeRowIndex);
            } else {
                orderRowIndex.add(size + orderRowIndex.get(orderRowIndex.size() - 1));
            }

            // 赋值sku无需合并的行
            for (DocumentByOrderVo.SkuSpec skuSpec : skuSpecList) {
                if (skuRowIndex.size() == 0) {
                    skuRowIndex.add(skuSpec.getBomSpecList().size() + mergeRowIndex);
                } else {
                    skuRowIndex.add(skuSpec.getBomSpecList().size() + skuRowIndex.get(skuRowIndex.size() - 1));
                }
            }

        }
    }

    @Override
    public void afterCellDispose(WriteSheetHolder writeSheetHolder, WriteTableHolder writeTableHolder, List<WriteCellData<?>> cellDataList, Cell cell, Head head, Integer relativeRowIndex, Boolean isHead) {

        // 当前行
        int curRowIndex = cell.getRowIndex();

        // 当前列
        int curColIndex = cell.getColumnIndex();

        // 前4行不合并
        if (curRowIndex <= mergeRowIndex) {
            return;
        }

        // 合并订单列
        if (orderColIndex.contains(curColIndex) && !orderRowIndex.contains(curRowIndex)) {
            mergeWithPrevRow(writeSheetHolder, curRowIndex, curColIndex);
        }

        // 合并sku列
        if (skuColIndex.contains(curColIndex) && !skuRowIndex.contains(curRowIndex)) {
            mergeWithPrevRow(writeSheetHolder, curRowIndex, curColIndex);
        }

    }

    /**
     * 当前单元格向上合并
     */
    private void mergeWithPrevRow(WriteSheetHolder writeSheetHolder, int curRowIndex, int curColIndex) {

        Sheet sheet = writeSheetHolder.getSheet();
        List<CellRangeAddress> mergeRegions = sheet.getMergedRegions();

        for (int i = 0; i < mergeRegions.size(); i++) {
            CellRangeAddress cellRangeAddr = mergeRegions.get(i);
            // 若上一个单元格已经被合并，则先移出原有的合并单元，再重新添加合并单元
            if (cellRangeAddr.isInRange(curRowIndex - 1, curColIndex)) {
                sheet.removeMergedRegion(i);
                cellRangeAddr.setLastRow(curRowIndex);
                sheet.addMergedRegion(cellRangeAddr);
                return;
            }
        }

        // 若上一个单元格未被合并，则新增合并单元
        CellRangeAddress cellRangeAddress = new CellRangeAddress(curRowIndex - 1, curRowIndex, curColIndex, curColIndex);
        sheet.addMergedRegion(cellRangeAddress);

    }

}
