package com.sd.business.service.statement.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.sd.business.entity.statement.bo.ExportDocumentByOrderBo;
import com.sd.business.entity.statement.vo.DocumentByBomVo;
import com.sd.business.entity.statement.vo.DocumentByOrderVo;
import com.sd.business.entity.statement.vo.DocumentBySkuVo;
import com.sd.business.service.statement.StatementOfAccountExportService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.framework.util.StreamUtil;
import com.sd.framework.util.TemplateExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class StatementOfAccountExportServiceImpl implements StatementOfAccountExportService {

    @Autowired
    private HttpServletResponse response;

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Override
    public void exportDocumentBySku(List<Long> idList, String departmentName, String beginDate, String endDate) {

        List<DocumentBySkuVo> list = statementOfAccountService.getDocumentBySku(idList);

        Map<String, Object> map = new HashMap<>();
        map.put("department", StrUtil.isBlank(departmentName) ? StringPool.EMPTY : departmentName + StringPool.DASH);
        map.put("beginDate", beginDate);
        map.put("endDate", endDate);
        map.put("totalQuantity", StreamUtil.bigDecimalAdd(list, DocumentBySkuVo::getQuantity));
        map.put("totalSubtotal", StreamUtil.bigDecimalAdd(list, DocumentBySkuVo::getSubtotal));

        TemplateExcelUtil.writeBrowser("skuDocument.xlsx", "sku对账单", response, list, map);
    }

    @Override
    public void exportDocumentByBom(List<Long> idList, String departmentName, String beginDate, String endDate) {

        List<DocumentByBomVo> list = statementOfAccountService.getDocumentByBom(idList);

        Map<String, Object> map = new HashMap<>();
        map.put("department", StrUtil.isBlank(departmentName) ? StringPool.EMPTY : departmentName + StringPool.DASH);
        map.put("beginDate", beginDate);
        map.put("endDate", endDate);
        map.put("totalSubtotal", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getSubtotal));
        map.put("totalLaserLogoSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getLaserLogoSummary));
        map.put("totalLaserMitochondrialSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getLaserMitochondrialSummary));
        map.put("totalLssueFeeSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getLssueFeeSummary));
        map.put("totalDeliveryMaterialsFeeSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getDeliveryMaterialsFeeSummary));
        map.put("totalPackingLaborSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getPackingLaborSummary));

        TemplateExcelUtil.writeBrowser("bomDocument.xlsx", "bom对账单", response, list, map);
    }

    @Override
    public void exportDocumentByOrder(List<Long> idList, String departmentName, String beginDate, String endDate) {

        List<DocumentByOrderVo> list = statementOfAccountService.getDocumentByOrder(idList);

        BigDecimal all = BigDecimal.ZERO;

        List<ExportDocumentByOrderBo> exportDocumentByOrderBos = new ArrayList<>();
        for (DocumentByOrderVo documentByOrderVo : list) {
            List<DocumentByOrderVo.SkuSpec> skuSpecList = documentByOrderVo.getSkuSpecList();
            for (DocumentByOrderVo.SkuSpec skuSpec : skuSpecList) {
                List<DocumentByOrderVo.BomSpec> bomSpecList = skuSpec.getBomSpecList();
                for (DocumentByOrderVo.BomSpec bomSpec : bomSpecList) {
                    ExportDocumentByOrderBo exportDocumentByOrderBo = ExportDocumentByOrderBo.builder()
                            .wlnCreateTime(documentByOrderVo.getWlnCreateTime())
                            .code(documentByOrderVo.getCode())
                            .wlnCode(documentByOrderVo.getWlnCode())
                            .skuSpecCode(skuSpec.getSkuSpecCode())
                            .skuSpecName(skuSpec.getSkuSpecName())
                            .quantity(skuSpec.getQuantity())
                            .unitPrice(skuSpec.getUnitPrice())
                            .subtotal(skuSpec.getSubtotal())
                            .total(skuSpec.getTotal())
                            .bomSpecCode(bomSpec.getBomSpecCode())
                            .bomSpecName(bomSpec.getBomSpecName())
                            .bomQuantity(bomSpec.getQuantity())
                            .bomUnitPrice(bomSpec.getUnitPrice())
                            .laserLogoSummary(bomSpec.getLaserLogoSummary())
                            .laserMitochondrialSummary(bomSpec.getLaserMitochondrialSummary())
                            .lssueFeeSummary(bomSpec.getLssueFeeSummary())
                            .deliveryMaterialsFeeSummary(bomSpec.getDeliveryMaterialsFeeSummary())
                            .packingLaborSummary(bomSpec.getPackingLaborSummary())
                            .build();
                    exportDocumentByOrderBos.add(exportDocumentByOrderBo);
                }
                all = all.add(skuSpec.getTotal());
            }
        }

        Map<String, Object> map = new HashMap<>();
        map.put("department", StrUtil.isBlank(departmentName) ? StringPool.EMPTY : departmentName + StringPool.DASH);
        map.put("beginDate", beginDate);
        map.put("endDate", endDate);
        map.put("all", all);

        TemplateExcelUtil.writeBrowser("orderDocument.xlsx", "订单对账单",
                new DocumentByOrderExcelCellMergeStrategy(list), response, exportDocumentByOrderBos, map);
    }

}
