package com.sd.business.service.statement.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.PageUtils;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.statement.dto.ExportDocumentDto;
import com.sd.business.entity.statement.dto.GetDocumentDto;
import com.sd.business.entity.statement.dto.StatementOfAccountMergePageDto;
import com.sd.business.entity.statement.po.StatementOfAccount;
import com.sd.business.entity.statement.vo.DocumentByBomVo;
import com.sd.business.entity.statement.vo.DocumentByOrderVo;
import com.sd.business.entity.statement.vo.DocumentBySkuVo;
import com.sd.business.entity.statement.vo.StatementOfAccountMergePageVo;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.statement.StatementOfAccountExportService;
import com.sd.business.service.statement.StatementOfAccountMergeService;
import com.sd.business.service.statement.StatementOfAccountService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 对账单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-31
 */
@Service
public class StatementOfAccountMergeServiceImpl implements StatementOfAccountMergeService {

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private StatementOfAccountExportService statementOfAccountExportService;


    @Override
    public Page<StatementOfAccountMergePageVo> getPage(StatementOfAccountMergePageDto dto) {

        QueryWrapper<StatementOfAccount> wrapper = Wrappers.query();

        wrapper.eq(ObjectUtil.isNotNull(dto.getDepartmentId()), "department_id", dto.getDepartmentId());
        wrapper.groupBy("department_id");
        wrapper.orderByDesc("dimensionality");

        switch (dto.getType()) {
            // 月度
            case 1:
                wrapper.select("department_id AS departmentId",
                        "GROUP_CONCAT(id) AS idGroupConcat",
                        "DATE_FORMAT(time_period,'%Y/%m月') AS dimensionality");
                wrapper.groupBy("dimensionality");
                break;

            // 季度
            case 2:
                wrapper.select("department_id AS departmentId", "GROUP_CONCAT(id) AS idGroupConcat",
                        "DATE_FORMAT(time_period,'%Y/%m月') AS dimensionality",
                        "CONCAT(YEAR(time_period),'/第',QUARTER(time_period),'季度') AS dimensionality");

                wrapper.groupBy("dimensionality");
                break;

            // 年度
            case 3:
                wrapper.select("department_id AS departmentId", "GROUP_CONCAT(id) AS idGroupConcat",
                        "DATE_FORMAT(time_period,'%Y/%m月') AS dimensionality",
                        "YEAR(time_period) AS dimensionality");
                wrapper.groupBy("dimensionality");
                break;

            // 自定义
            case 4:
                wrapper.select("department_id AS departmentId", "GROUP_CONCAT(id) AS idGroupConcat",
                        "DATE_FORMAT(time_period,'%Y/%m月') AS dimensionality");
                wrapper.ge(ObjectUtil.isNotNull(dto.getBeginTime()), "time_period", dto.getBeginTime());
                wrapper.le(ObjectUtil.isNotNull(dto.getEndTime()), "time_period", dto.getEndTime());
                break;

            default:
                throw new ServiceException("未知对账类型");
        }

        Page<Map<String, Object>> mapPage = statementOfAccountService.pageMaps(dto.getPage(), wrapper);
        Page<StatementOfAccountMergePageVo> page = PageUtils.copyPage(mapPage, StatementOfAccountMergePageVo.class);
        List<StatementOfAccountMergePageVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        departmentService.attributeAssign(records, StatementOfAccountMergePageVo::getDepartmentId,
                (item, department) -> item.setDepartmentName(department.getName()));

        // 对账单id列表
        List<Long> statementOfAccountIdList = records.stream()
                .map(StatementOfAccountMergePageVo::getIdGroupConcat)
                .flatMap(item -> Arrays.stream(item.split(",")))
                .map(Convert::toLong)
                .distinct()
                .collect(Collectors.toList());

        Map<Long, List<OrderInfo>> map = orderService.mapKGroup(OrderInfo::getStatementOfAccountId,
                q -> q.in(OrderInfo::getStatementOfAccountId, statementOfAccountIdList));

        for (StatementOfAccountMergePageVo record : records) {

            if (dto.getType().equals(4)) {
                String beginTime = dto.getBeginTime() == null ? StringPool.EMPTY : DateUtil.formatDate(dto.getBeginTime());
                String endTime = dto.getEndTime() == null ? StringPool.EMPTY : DateUtil.formatDate(dto.getEndTime());
                record.setDimensionality(beginTime + StringPool.DASH + endTime);
            }

            String[] tempStatementOfAccountIdList = record.getIdGroupConcat().split(",");

            record.setReconcilingAmount(BigDecimal.ZERO);
            record.setOrderQuantity(0);
            record.setQuantityOfStatement(tempStatementOfAccountIdList.length);

            for (String statementOfAccountIdStr : tempStatementOfAccountIdList) {
                Long statementOfAccountId = Convert.toLong(statementOfAccountIdStr);
                List<OrderInfo> tempOrderInfoList = map.get(statementOfAccountId);

                if (ObjectUtil.isEmpty(tempOrderInfoList)) {
                    continue;
                }

                BigDecimal reconcilingAmount = tempOrderInfoList.stream()
                        .map(OrderInfo::getTotalAmount).reduce(BigDecimal.ZERO, BigDecimal::add);

                record.setOrderQuantity(record.getOrderQuantity() + tempOrderInfoList.size());
                record.setReconcilingAmount(record.getReconcilingAmount().add(reconcilingAmount));
            }

        }

        return page;
    }

    @Override
    public List<DocumentBySkuVo> getDocumentBySku(GetDocumentDto dto) {
        return statementOfAccountService.getDocumentBySku(getIdList(dto));
    }

    @Override
    public List<DocumentByBomVo> getDocumentByBom(GetDocumentDto dto) {
        return statementOfAccountService.getDocumentByBom(getIdList(dto));
    }

    @Override
    public List<DocumentByOrderVo> getDocumentByOrder(GetDocumentDto dto) {
        return statementOfAccountService.getDocumentByOrder(getIdList(dto));
    }

    @Override
    public void export(ExportDocumentDto dto, int type) {

        List<Long> idList = Arrays.stream(dto.getIdGroupConcat().split(","))
                .map(Convert::toLong)
                .filter(Objects::nonNull)
                .distinct()
                .collect(Collectors.toList());

        String departmentName = dto.getDepartmentName();
        String beginDate = dto.getBeginDate();
        String endDate = dto.getEndDate();

        switch (type) {
            case 1:
                statementOfAccountExportService.exportDocumentBySku(idList, departmentName, beginDate, endDate);
                break;
            case 2:
                statementOfAccountExportService.exportDocumentByBom(idList, departmentName, beginDate, endDate);
                break;
            case 3:
                statementOfAccountExportService.exportDocumentByOrder(idList, departmentName, beginDate, endDate);
                break;
            default:
                throw new ServiceException("未知对账单类型");
        }
    }

    private List<Long> getIdList(GetDocumentDto dto) {

        String idGroupConcat = dto.getIdGroupConcat();

        List<Long> idList = Arrays.stream(idGroupConcat.split(","))
                .map(Convert::toLong)
                .distinct()
                .filter(Objects::nonNull)
                .collect(Collectors.toList());

        Assert.notEmpty(idList, "对账单id集合不能为空");

        return idList;
    }

}
