import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.DynamicRoutingDataSource;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.sd.SdApplication;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.sku.SkuSpecService;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.test.context.junit4.SpringRunner;

import javax.sql.DataSource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 同步sku关联主材
 */
@RunWith(SpringRunner.class)
@SpringBootTest(classes = SdApplication.class, webEnvironment = SpringBootTest.WebEnvironment.DEFINED_PORT)
public class A2_SyncSkuTest {

    @Autowired
    private DataSource dataSource;
    @Autowired
    private SkuSpecService skuSpecService;
    @Autowired
    private BomSpecService bomSpecService;

    @DSTransactional
    @Test
    public void test() {

        DynamicRoutingDataSource dynamicRoutingDataSource = (DynamicRoutingDataSource) dataSource;
        DataSource sdDataSource = dynamicRoutingDataSource.getDataSource("business");

        Map<String, Long> bomSpecIdMap = bomSpecService.mapKV(BomSpec::getOldId, BaseIdPo::getId, null);

        Map<String, Long> stringLongMap = skuSpecService.mapKV(SkuSpec::getCode, BaseIdPo::getId, null);

        String sql = "SELECT\n" +
                "\tp.spec_code code,\n" +
                "\tp.weight netWeight,\n" +
                "\tp.length length,\n" +
                "\tp.width width,\n" +
                "\tp.height height,\n" +
                "\tp.processing_layout machinedPanel,\n" +
                "\ts.bom_color_id bomSpecId\n" +
                "FROM\n" +
                "\tt_sd_product_color p\n" +
                "\tLEFT JOIN t_sd_product_color_bom s on p.id = s.product_color_id";

        List<Map<String, Object>> list = new JdbcTemplate(sdDataSource).queryForList(sql);

        for (Map<String, Object> map : list) {
            Object bomSpecIdObj = map.get("bomSpecId");
            if (bomSpecIdObj == null) {
                continue;
            }
            String bomSpecId = bomSpecIdObj.toString();
            if (StrUtil.isBlank(bomSpecId)) {
                continue;
            }
            map.put("bomSpecId", bomSpecIdMap.get(bomSpecId));
        }
        List<SkuSpec> skuSpecList = BeanUtil.copyToList(list, SkuSpec.class);

        for (SkuSpec skuSpec : skuSpecList) {
            String code = skuSpec.getCode();
            Long id = stringLongMap.get(code);
            skuSpec.setId(id);
        }

        skuSpecList = skuSpecList.stream().filter(item -> ObjectUtil.isNotNull(item.getId())).collect(Collectors.toList());

        skuSpecService.updateBatchById(skuSpecList);

    }

}